USE master
GO

-- Create a login
CREATE LOGIN HRLogin WITH PASSWORD = 'HRp@$$w0rd'
GO 

-- Create a new database
IF EXISTS(SELECT name FROM sys.databases WHERE name = 'MyDB')
 DROP DATABASE MyDB 
GO
CREATE DATABASE MyDB
GO
USE MyDB
GO

-- Create a user mapped to the login
CREATE USER HRUser FOR LOGIN HRLogin 
GO 
	 
-- Create the database master key (DMK)
CREATE MASTER KEY ENCRYPTION BY PASSWORD = 'DMKp@$$w0rd' 
GO 

-- Create a table that will store sensitive credit card numbers. Notice
-- the varbinary data type that will store the encrypted card numbers.
CREATE TABLE CreditCardInfo
(CardType varchar(50),
 CardNumber varbinary(255)) 
GO 
	 
-- Give access to HRUser so they can query and add data
GRANT SELECT, INSERT TO HRUser
GO 
	 
-- Create a Symmetric Key, encrypt it with a password, and give HRUser access to it
CREATE SYMMETRIC KEY HRUserKey
 AUTHORIZATION HRUser 
 WITH ALGORITHM = TRIPLE_DES 
 ENCRYPTION BY PASSWORD = 'SYMp@$$w0rd'
GO 
	 
-- Impersonate HRUser and encrypt some data 
EXECUTE AS LOGIN = 'HRLogin'
GO 
	 
-- Open the key that will be used to encrypt data. Notice you have to supply
-- the password for the key 
OPEN SYMMETRIC KEY HRUserKey DECRYPTION BY PASSWORD = 'SYMp@$$w0rd'
GO 
	 
-- This system view shows open keys that can be used for encryption 
SELECT * FROM sys.openkeys 
	 
-- Insert sensitive data into the table using ENCRYPTBYKEY, which takes the
-- GUID of the key (using KEY_GUID) and the text to be encrypted, and returns
-- the result as varbinary.
INSERT INTO CreditCardInfo VALUES
 ('MasterCard', ENCRYPTBYKEY(KEY_GUID('HRUserKey'), '5426-1891-5411-1369')),
 ('American Express', ENCRYPTBYKEY(KEY_GUID('HRUserKey'), '3728-847852-83004'))
 
-- When done, always close all keys 
CLOSE ALL SYMMETRIC KEYS 
GO 
 
-- View the table as it lives in the database, notice the encrypted binary
SELECT * FROM CreditCardInfo 
 
-- Now, decrypt and view the contents using DECRYPTBYKEY, which takes the column
-- name. You don't specify a key GUID because SQL will look at all open keys and
-- use the appropriate one automatically.
OPEN SYMMETRIC KEY HRUserKey DECRYPTION BY PASSWORD = 'SYMp@$$w0rd'
GO 

SELECT *, CONVERT(varchar, DECRYPTBYKEY(CardNumber))  
 FROM CreditCardInfo 
GO 
	 
CLOSE ALL SYMMETRIC KEYS 
GO 
	 
-- Revert back to sysadmin
REVERT 
GO 
 
-- When encrypting by password, need to know the password and pass it every time
-- you encrypt/decrypt. Alternatively you can create a certificate and give access
-- to the HR User. With this, the user doesn't have to provide a password and you
-- can easily revoke access to that encrypted data by simply removing the
-- certificate for that use.
CREATE CERTIFICATE HRCert
 AUTHORIZATION HRUser 
 WITH SUBJECT = 'Certificate used by the Human Resources person'
 
-- Open the key so you can modify it 
OPEN SYMMETRIC KEY HRUserKey DECRYPTION BY PASSWORD = 'SYMp@$$w0rd'
GO 
 
-- You cannot remove the password because that would leave the key 
-- exposed without encryption, so you need to add the certificate first 
ALTER SYMMETRIC KEY HRUserKey 
 ADD ENCRYPTION BY CERTIFICATE HRCert
GO 
 
-- Now you can remove the password encryption from the key  
ALTER SYMMETRIC KEY HRUserKey 
 DROP ENCRYPTION BY PASSWORD = 'SYMp@$$w0rd'
GO 

CLOSE ALL SYMMETRIC KEYS 
GO 
 
-- Now change context to HRLogin to test
EXECUTE AS LOGIN = 'HRLogin' 
GO 
 
-- Notice the key is opened without a password this time, because you created
-- the certificate and gave authorization on it explicitly to HRUser.
OPEN SYMMETRIC KEY HRUserKey DECRYPTION BY CERTIFICATE HRCert
GO 
 
SELECT *, CONVERT(varchar, DECRYPTBYKEY(CardNumber))  
 FROM CreditCardInfo 
GO 
	 
CLOSE ALL SYMMETRIC KEYS 
GO 
	 
-- Revert back to sysadmin
REVERT 
GO
